@@ -2,20 +2,22 @@
 #this repository contains the full copyright notices and license terms.
 
 import copy
+import collections
 from trytond.model import fields
+from trytond.error import WarningErrorMixin
 from trytond.pool import Pool
 from trytond.pyson import PYSONEncoder
 from trytond.transaction import Transaction
+from trytond.url import URLMixin
 
 
-class Model(object):
+class Model(WarningErrorMixin, URLMixin):
     """
     Define a model in Tryton.
     """
     _name = None
     _inherits = {}
     _description = ''
-    pool = None #XXX change to avoid collision with field
     __columns = None
     __xxx2many_targets = None
     __defaults = None
@@ -52,8 +54,9 @@ class Model(object):
         fields_names = self._columns.keys()
         fields_names += self._inherit_fields.keys()
         for field_name in fields_names:
-            if getattr(self, 'default_' + field_name, False):
-                res[field_name] = getattr(self, 'default_' + field_name)
+            default_method = getattr(self, 'default_%s' % field_name, False)
+            if isinstance(default_method, collections.Callable):
+                res[field_name] = default_method
         self.__defaults = res
         return res
 
@@ -82,7 +85,9 @@ class Model(object):
             assert (k in self._columns) or (k in self._inherit_fields), \
             'Default function defined in %s but field %s does not exist!' % \
                 (self._name, k,)
+        self._update_rpc()
 
+    def _update_rpc(self):
         for field_name in self._columns.keys() + self._inherit_fields.keys():
             if field_name in self._columns:
                 field = self._columns[field_name]
@@ -92,58 +97,45 @@ class Model(object):
                     and not isinstance(field.selection, (list, tuple)) \
                     and field.selection not in self._rpc:
                 self._rpc[field.selection] = False
-            if field.on_change:
-                on_change = 'on_change_' + field_name
-                if on_change not in self._rpc:
-                    self._rpc[on_change] = False
-            if field.on_change_with:
-                on_change_with = 'on_change_with_' + field_name
-                if on_change_with not in self._rpc:
-                    self._rpc[on_change_with] = False
+
+            for attribute in ('on_change', 'on_change_with', 'autocomplete'):
+                function_name = '%s_%s' % (attribute, field_name)
+                if (getattr(field, attribute, False)
+                         and isinstance(getattr(self, function_name, False),
+                             collections.Callable)):
+                    self._rpc.setdefault(function_name, False)
 
     def __getattr__(self, name):
+        pool = Pool()
         # Search if a function exists in inherits parents
         for model_name, field_name in self._inherits.iteritems():
-            model_obj = self.pool.get(model_name)
-            if hasattr(model_obj, name) and \
-                    callable(getattr(model_obj, name)):
-                def func(*args, **kwargs):
-                    if args:
-                        ids = args[0]
-                        int_id = False
-                        if isinstance(ids, (int, long)):
-                            int_id = True
-                            ids = [ids]
-                        if reduce(lambda x, y: x and
-                                isinstance(y, (int, long)), ids, True):
-                            # Replace ids by the parent ids
-                            ids = [getattr(x, field_name).id for x in
-                                    self.browse(ids)]
-                        if int_id:
-                            ids = ids[0]
-                        args = tuple([ids] + list(args[1:]))
-                    return getattr(model_obj, name)(*args, **kwargs)
-                return func
-        raise AttributeError
+            model_obj = pool.get(model_name)
+            if (hasattr(model_obj, name)
+                    and isinstance(getattr(model_obj, name),
+                        collections.Callable)):
+               return getattr(model_obj, name)
+        raise AttributeError(name)
 
     def _inherits_reload(self):
         """
         Reconstruct _inherit_fields
         """
         res = {}
+        pool = Pool()
         for model in self._inherits:
-            res.update(self.pool.get(model)._inherit_fields)
-            for field_name in self.pool.get(model)._columns.keys():
+            res.update(pool.get(model)._inherit_fields)
+            for field_name in pool.get(model)._columns.keys():
                 res[field_name] = (model, self._inherits[model],
-                        self.pool.get(model)._columns[field_name])
-            for field_name in self.pool.get(model)._inherit_fields.keys():
+                        pool.get(model)._columns[field_name])
+            for field_name in pool.get(model)._inherit_fields.keys():
                 res[field_name] = (model, self._inherits[model],
-                        self.pool.get(model)._inherit_fields[field_name][2])
+                        pool.get(model)._inherit_fields[field_name][2])
         self._inherit_fields = res
-        self._reset_xxx2many_targets()
+        self._reset_columns()
+        self._update_rpc()
         # Update objects that uses this one to update their _inherits fields
-        for obj_name in self.pool.object_name_list():
-            obj = self.pool.get(obj_name)
+        for obj_name in pool.object_name_list():
+            obj = pool.get(obj_name)
             if self._name in obj._inherits:
                 obj._inherits_reload()
 
@@ -164,7 +156,7 @@ class Model(object):
                 continue
             if hasattr(field, 'get_target'):
                 try:
-                    model_name = field.get_target(self.pool)._name
+                    model_name = field.get_target()._name
                 except KeyError:
                     to_cache = False
                     continue
@@ -182,7 +174,7 @@ class Model(object):
                 continue
             if hasattr(field, 'get_target'):
                 try:
-                    model_name = field.get_target(self.pool)._name
+                    model_name = field.get_target()._name
                 except KeyError:
                     to_cache = False
                     continue
@@ -202,6 +194,7 @@ class Model(object):
 
         :param module_name: the module name
         """
+        translation_obj = Pool().get('ir.translation')
 
         cursor = Transaction().cursor
         # Add model in ir_model
@@ -244,16 +237,18 @@ class Model(object):
                     trans_id, = data
             elif cursor.rowcount != 0:
                 trans_id, = cursor.fetchone()
+            src_md5 = translation_obj.get_src_md5(src)
             if not trans_id:
-                cursor.execute('INSERT INTO ir_translation ' \
-                        '(name, lang, type, src, value, module, fuzzy) ' \
-                        'VALUES (%s, %s, %s, %s, %s, %s, %s)',
-                        (name, 'en_US', 'model', src, '', module_name, False))
+                cursor.execute('INSERT INTO ir_translation '
+                    '(name, lang, type, src, src_md5, value, module, fuzzy) '
+                    'VALUES (%s, %s, %s, %s, %s, %s, %s, %s)',
+                    (name, 'en_US', 'model', src, src_md5, '', module_name,
+                        False))
             else:
-                cursor.execute('UPDATE ir_translation ' \
-                        'SET src = %s ' \
-                        'WHERE id = %s',
-                        (src, trans_id))
+                cursor.execute('UPDATE ir_translation '
+                    'SET src = %s, src_md5 = %s '
+                    'WHERE id = %s',
+                        (src, src_md5, trans_id))
 
         # Add field in ir_model_field and update translation
         cursor.execute('SELECT f.id AS id, f.name AS name, ' \
@@ -317,31 +312,33 @@ class Model(object):
                         (field.string, field._type, relation,
                             field.help, model_fields[field_name]['id']))
             trans_name = self._name + ',' + field_name
+            string_md5 = translation_obj.get_src_md5(field.string)
             if trans_name not in trans_fields:
                 if field_name not in ('create_uid', 'create_date',
                             'write_uid', 'write_date', 'id'):
-                    cursor.execute('INSERT INTO ir_translation ' \
-                            '(name, lang, type, src, value, module, fuzzy) ' \
-                            'VALUES (%s, %s, %s, %s, %s, %s, %s)',
-                            (trans_name, 'en_US', 'field',
-                                field.string, '', module_name, False))
+                    cursor.execute('INSERT INTO ir_translation '
+                        '(name, lang, type, src, src_md5, value, module, fuzzy) '
+                        'VALUES (%s, %s, %s, %s, %s, %s, %s, %s)',
+                        (trans_name, 'en_US', 'field', field.string,
+                            string_md5, '', module_name, False))
             elif trans_fields[trans_name]['src'] != field.string:
-                cursor.execute('UPDATE ir_translation ' \
-                        'SET src = %s ' \
-                        'WHERE id = %s ',
-                        (field.string, trans_fields[trans_name]['id']))
+                cursor.execute('UPDATE ir_translation '
+                    'SET src = %s, src_md5 = %s '
+                    'WHERE id = %s ',
+                    (field.string, string_md5, trans_fields[trans_name]['id']))
+            help_md5 = translation_obj.get_src_md5(field.help)
             if trans_name not in trans_help:
                 if field.help:
-                    cursor.execute('INSERT INTO ir_translation ' \
-                            '(name, lang, type, src, value, module, fuzzy) ' \
-                            'VALUES (%s, %s, %s, %s, %s, %s, %s)',
-                            (trans_name, 'en_US', 'help',
-                                field.help, '', module_name, False))
+                    cursor.execute('INSERT INTO ir_translation '
+                        '(name, lang, type, src, src_md5, value, module, fuzzy) '
+                        'VALUES (%s, %s, %s, %s, %s, %s, %s, %s)',
+                        (trans_name, 'en_US', 'help', field.help, help_md5, '',
+                            module_name, False))
             elif trans_help[trans_name]['src'] != field.help:
-                cursor.execute('UPDATE ir_translation ' \
-                        'SET src = %s ' \
-                        'WHERE id = %s ',
-                        (field.help, trans_help[trans_name]['id']))
+                cursor.execute('UPDATE ir_translation '
+                    'SET src = %s, src_md5 = %s '
+                    'WHERE id = %s ',
+                    (field.help, help_md5, trans_help[trans_name]['id']))
             if hasattr(field, 'selection') \
                     and isinstance(field.selection, (tuple, list)) \
                     and ((hasattr(field, 'translate_selection') \
@@ -350,12 +347,13 @@ class Model(object):
                 for (_, val) in field.selection:
                     if trans_name not in trans_selection \
                             or val not in trans_selection[trans_name]:
-                        cursor.execute('INSERT INTO ir_translation ' \
-                                '(name, lang, type, src, value, ' \
-                                    'module, fuzzy) ' \
-                                'VALUES (%s, %s, %s, %s, %s, %s, %s)',
-                                (trans_name, 'en_US', 'selection', val, '',
-                                    module_name, False))
+                        val_md5 = translation_obj.get_src_md5(val)
+                        cursor.execute('INSERT INTO ir_translation '
+                            '(name, lang, type, src, src_md5, value, module, '
+                                'fuzzy) '
+                            'VALUES (%s, %s, %s, %s, %s, %s, %s, %s)',
+                            (trans_name, 'en_US', 'selection', val, val_md5,
+                                '', module_name, False))
         # Clean ir_model_field from field that are no more existing.
         for field_name in model_fields:
             if model_fields[field_name]['module'] == module_name \
@@ -379,117 +377,16 @@ class Model(object):
         errors = self._get_error_messages()
         for error in set(errors):
             if error not in trans_error:
-                cursor.execute('INSERT INTO ir_translation ' \
-                        '(name, lang, type, src, value, module, fuzzy) ' \
-                        'VALUES (%s, %s, %s, %s, %s, %s, %s)',
-                        (self._name, 'en_US', 'error', error, '', module_name,
-                            False))
+                error_md5 = translation_obj.get_src_md5(error)
+                cursor.execute('INSERT INTO ir_translation '
+                    '(name, lang, type, src, src_md5, value, module, fuzzy) '
+                    'VALUES (%s, %s, %s, %s, %s, %s, %s, %s)',
+                    (self._name, 'en_US', 'error', error, error_md5, '',
+                        module_name, False))
 
     def _get_error_messages(self):
         return self._error_messages.values()
 
-    def raise_user_error(self, error, error_args=None,
-            error_description='', error_description_args=None,
-            raise_exception=True):
-        '''
-        Raise an exception that will be displayed as an error message
-        in the client.
-
-        :param error: the key of the dictionary _error_messages used
-            for error message
-        :param error_args: the arguments that will be used
-            for "%"-based substitution
-        :param error_description: the key of the dictionary
-            _error_messages used for error description
-        :param error_description_args: the arguments that will be used
-            for "%"-based substitution
-        :param raise_exception: if set to False return the error string
-            (or tuple if error_description is not empty) instead of raising an
-            exception.
-        '''
-        translation_obj = self.pool.get('ir.translation')
-
-        error = self._error_messages.get(error, error)
-
-        language = Transaction().context.get('language') or 'en_US'
-        res = translation_obj._get_source(self._name, 'error', language, error)
-        if not res:
-            res = translation_obj._get_source(error, 'error', language)
-        if not res:
-            res = translation_obj._get_source(error, 'error', 'en_US')
-
-        if res:
-            error = res
-
-        if error_args:
-            try:
-                error = error % error_args
-            except TypeError:
-                pass
-
-        if error_description:
-            error_description = self._error_messages.get(error_description,
-                    error_description)
-
-            res = translation_obj._get_source(self._name, 'error', language,
-                    error_description)
-            if not res:
-                res = translation_obj._get_source(error_description, 'error',
-                        language)
-            if not res:
-                res = translation_obj._get_source(error_description, 'error',
-                        'en_US')
-
-            if res:
-                error_description = res
-
-            if error_description_args:
-                try:
-                    error_description = error_description % \
-                            error_description_args
-                except TypeError:
-                    pass
-            if raise_exception:
-                raise Exception('UserError', error, error_description)
-            else:
-                return (error, error_description)
-        if raise_exception:
-            raise Exception('UserError', error)
-        else:
-            return error
-
-    def raise_user_warning(self, warning_name, warning,
-            warning_args=None, warning_description='',
-            warning_description_args=None):
-        '''
-        Raise an exception that will be displayed as a warning message
-        in the client, if the user has not yet bypassed it.
-
-        :param warning_name: the unique warning name
-        :param warning: the key of the dictionary _error_messages used
-            for warning message
-        :param warning_args: the arguments that will be used for
-            "%"-based substitution
-        :param warning_description: the key of the dictionary
-            _error_messages used for warning description
-        :param warning_description_args: the arguments that will be used
-            for "%"-based substitution
-        '''
-        warning_obj = self.pool.get('res.user.warning')
-        if warning_obj.check(warning_name):
-            if warning_description:
-                warning, warning_description = self.raise_user_error(warning,
-                        error_args=warning_args,
-                        error_description=warning_description,
-                        error_description_args=warning_description_args,
-                        raise_exception=False)
-                raise Exception('UserWarning', warning_name, warning,
-                        warning_description)
-            else:
-                warning = self.raise_user_error(warning,
-                        error_args=warning_args, raise_exception=False)
-                raise Exception('UserWarning', warning_name, warning)
-
     def default_get(self, fields_names, with_rec_name=True):
         '''
         Return a dict with the default values for each field in fields_names.
@@ -499,8 +396,8 @@ class Model(object):
         :return: a dictionary with field name as key
             and default value as value
         '''
-        property_obj = self.pool.get('ir.property')
-        ir_default_obj = self.pool.get('ir.default')
+        pool = Pool()
+        property_obj = pool.get('ir.property')
         value = {}
 
         # get the default values defined in the object
@@ -519,75 +416,11 @@ class Model(object):
             if (with_rec_name
                     and field._type in ('many2one',)
                     and value.get(field_name)):
-                obj = self.pool.get(field.model_name)
+                obj = pool.get(field.model_name)
                 if 'rec_name' in obj._columns:
                     value[field_name + '.rec_name'] = obj.browse(
                         value[field_name]).rec_name
 
-        # get the default values set by the user and override the default
-        # values defined in the object
-        for name in (self._inherits.keys() + [self._name]):
-            defaults = ir_default_obj.get_default(name, False)
-            for field_name, field_value in defaults.items():
-                if field_name in fields_names:
-                    if field_name in self._columns:
-                        field = self._columns[field_name]
-                    else:
-                        field = self._inherit_fields[field_name][2]
-                    if field._type in ('many2one',):
-                        if not isinstance(field_value, (int, long)):
-                            continue
-                        obj = self.pool.get(field.model_name)
-                        if not hasattr(obj, 'search') \
-                                or not obj.search([
-                                    ('id', '=', field_value),
-                                    ]):
-                            continue
-                        if with_rec_name and 'rec_name' in obj._columns:
-                            value[field_name + '.rec_name'] = obj.browse(
-                                    field_value).rec_name
-                    if field._type in ('many2many'):
-                        if not isinstance(field_value, list):
-                            continue
-                        obj = field.get_target(self.pool)
-                        field_value2 = []
-                        for i in range(len(field_value)):
-                            if not hasattr(obj, 'search') \
-                                    or not obj.search([
-                                        ('id', '=', field_value[i]),
-                                        ]):
-                                continue
-                            field_value2.append(field_value[i])
-                        field_value = field_value2
-                    if field._type in ('one2many'):
-                        if not isinstance(field_value, list):
-                            continue
-                        obj = self.pool.get(field.model_name)
-                        field_value2 = []
-                        for i in range(len(field_value or [])):
-                            field_value2.append({})
-                            for field2 in field_value[i].keys():
-                                if (field2 in obj._columns
-                                        and obj._columns[field2]._type
-                                        in ('many2one',)):
-                                    obj2 = self.pool.get(
-                                            obj._columns[field2].model_name)
-                                    if not hasattr(obj2, 'search'):
-                                        continue
-                                    if not obj2.search([
-                                        ('id', '=', field_value[i][field2]),
-                                        ]):
-                                        continue
-                                    if (with_rec_name
-                                            and 'rec_name' in obj2._columns):
-                                        field_value[i][field2 + '.rec_name'] = (
-                                                obj2.browse(
-                                                    field_value[i][field2],
-                                                    ).rec_name)
-                                # TODO add test for many2many and one2many
-                                field_value2[i][field2] = field_value[i][field2]
-                        field_value = field_value2
-                    value[field_name] = field_value
         value = self._default_on_change(value)
         if not with_rec_name:
             for field in value.keys():
@@ -603,10 +436,11 @@ class Model(object):
         :param value: a dictionnary with the default value
         :return: a new dictionnary of default value
         """
+        pool = Pool()
         res = value.copy()
         val = {}
         for i in self._inherits.keys():
-            val.update(self.pool.get(i)._default_on_change(value))
+            val.update(pool.get(i)._default_on_change(value))
         for field in value.keys():
             if field in self._columns:
                 if self._columns[field].on_change:
@@ -619,7 +453,7 @@ class Model(object):
                                 args[arg] = args[arg][0]
                     val.update(getattr(self, 'on_change_' + field)(args))
                 if self._columns[field]._type in ('one2many',):
-                    obj = self.pool.get(self._columns[field].model_name)
+                    obj = pool.get(self._columns[field].model_name)
                     for val2 in res[field]:
                         val2.update(obj._default_on_change(val2))
         res.update(val)
@@ -633,10 +467,13 @@ class Model(object):
         :return: a dictionary with field name as key and definition as value
         """
         res = {}
-        translation_obj = self.pool.get('ir.translation')
-        model_access_obj = self.pool.get('ir.model.access')
+        pool = Pool()
+        translation_obj = pool.get('ir.translation')
+        model_access_obj = pool.get('ir.model.access')
+        field_access_obj = pool.get('ir.model.field.access')
+
         for parent in self._inherits:
-            res.update(self.pool.get(parent).fields_get(fields_names))
+            res.update(pool.get(parent).fields_get(fields_names))
         write_access = model_access_obj.check(self._name, 'write',
                 raise_exception=False)
 
@@ -664,9 +501,14 @@ class Model(object):
 
         encoder = PYSONEncoder()
 
+        fwrite_accesses = field_access_obj.check(self._name, fields_names or
+                self._columns.keys(), 'write', access=True)
         for field in (x for x in self._columns.keys()
                 if ((not fields_names) or x in fields_names)):
-            res[field] = {'type': self._columns[field]._type}
+            res[field] = {
+                'type': self._columns[field]._type,
+                'name': field,
+                }
             for arg in (
                     'string',
                     'readonly',
@@ -680,13 +522,16 @@ class Model(object):
                     'on_change',
                     'add_remove',
                     'on_change_with',
+                    'autocomplete',
                     'sort',
                     'datetime_field',
+                    'loading',
+                    'filename',
                     ):
-                if getattr(self._columns[field], arg, None) != None:
+                if getattr(self._columns[field], arg, None) is not None:
                     res[field][arg] = copy.copy(getattr(self._columns[field],
                         arg))
-            if not write_access:
+            if not write_access or not fwrite_accesses.get(field, True):
                 res[field]['readonly'] = True
                 if res[field].get('states') and \
                         'readonly' in res[field]['states']:
@@ -700,6 +545,13 @@ class Model(object):
                     (fields.Function, fields.One2Many)) \
                     and not self._columns[field].order_field:
                 res[field]['sortable'] = False
+            if ((isinstance(self._columns[field], fields.Function)
+                    and not self._columns[field].searcher)
+                    or self._columns[field]._type in ('binary', 'many2many',
+                        'one2many', 'sha')):
+                res[field]['searchable'] = False
+            else:
+                res[field]['searchable'] = True
 
             if Transaction().context.get('language'):
                 # translate the field label
@@ -740,12 +592,13 @@ class Model(object):
                     'one2many',
                     'many2many',
                     'many2one',
+                    'one2one',
                     ):
                 if hasattr(self._columns[field], 'model_name'):
                     relation = copy.copy(self._columns[field].model_name)
                 else:
-                    relation = copy.copy(self._columns[field].get_target(
-                        self.pool)._name)
+                    relation = copy.copy(
+                        self._columns[field].get_target()._name)
                 res[field]['relation'] = relation
                 res[field]['domain'] = copy.copy(self._columns[field].domain)
                 res[field]['context'] = copy.copy(self._columns[field].context)
